import React from "react";
import { ChevronIcon } from "@orbit-ui/icons";
import { KEYS, mergeClasses, withHandlerProxy } from "@orbit-ui/react-components-shared";
import { PureComponent, createRef } from "react";
import { ResizeObserver } from "../resize-observer";
import { bool, func, node, string } from "prop-types";
import { isNil } from "lodash";
import { momentObj as momentType } from "react-moment-proptypes";
export class InlineSingleDatePickerInput extends PureComponent {
  constructor(...args) {
    super(...args);
    this._containerRef = createRef();

    this.handleContainerSizeChange = entries => {
      const {
        onSizeChange
      } = this.props;

      if (!isNil(onSizeChange)) {
        // The native chrome implementation doesn't currently support the "border-box" specs. Therefore, we rely on "getBoundingClientRect"
        // for the value when a size changed is detected.
        // Specs available here: https://drafts.csswg.org/resize-observer-1/
        const dimensions = entries[0].target.getBoundingClientRect();
        onSizeChange({
          width: dimensions.width,
          height: dimensions.height
        });
      }
    };

    this.handleClick = event => {
      const {
        onClick,
        onOpen,
        onClose,
        disabled,
        open
      } = this.props;

      if (!isNil(onClick)) {
        onClick(event, this.props);
      }

      if (!disabled) {
        if (!open) {
          onOpen(event, this.props);
        } else {
          onClose(event, this.props);
        }
      }
    };

    this.handleKeyDown = event => {
      const {
        onKeyDown,
        onOpen,
        onClear,
        disabled,
        open
      } = this.props;

      if (!isNil(onKeyDown)) {
        onKeyDown(event, this.props);
      }

      const key = event.keyCode;

      if (key === KEYS.space || key === KEYS.enter) {
        if (key === KEYS.space) {
          event.preventDefault();
        }

        if (!disabled) {
          if (!open) {
            onOpen(event, this.props);
          }
        }
      } else if (key === KEYS.esc) {
        if (!open) {
          onClear(event, this.props);
        }
      }
    };

    this.handleFocus = withHandlerProxy(this, "onFocus");
    this.handleBlur = withHandlerProxy(this, "onBlur");
  }

  componentDidMount() {
    this._containerResizeObserver = new ResizeObserver(this.handleContainerSizeChange);

    this._containerResizeObserver.observe(this._containerRef.current);
  }

  componentWillUnmount() {
    this._containerResizeObserver.disconnect();
  }

  getValue() {
    const {
      date,
      placeholder,
      dateFormat
    } = this.props;

    if (!isNil(date)) {
      return date.format(dateFormat);
    }

    return placeholder;
  }

  getCssClasses() {
    const {
      disabled,
      open,
      className
    } = this.props;
    return mergeClasses("flex items-center outline-0", open ? "bb bw1 b--primary-500" : "bw0 b--transparent", !disabled ? "primary-500 bb bw1 b--transparent hover-b--primary-500" : "cloud-200 hover-b--transparent crsr-not-allowed", className);
  }

  renderIcon() {
    const {
      openIcon,
      closeIcon,
      disabledCloseIcon,
      open,
      disabled
    } = this.props;

    if (open) {
      return openIcon;
    }

    return disabled ? disabledCloseIcon : closeIcon;
  }

  render() {
    const {
      disabled
    } = this.props;
    return React.createElement("div", {
      onClick: this.handleClick,
      onKeyDown: this.handleKeyDown,
      onFocus: this.handleFocus,
      onBlur: this.handleBlur,
      className: this.getCssClasses(),
      tabIndex: disabled ? "-1" : "0",
      disabled: disabled,
      ref: this._containerRef
    }, React.createElement("span", {
      className: "mr2 fw5"
    }, this.getValue()), React.createElement("span", {
      className: "flex"
    }, this.renderIcon()));
  } // This method is part of the component external API.


  focus() {
    if (!isNil(this._containerRef.current)) {
      this._containerRef.current.focus();
    }
  }

}
InlineSingleDatePickerInput.displayName = "InlineSingleDatePickerInput";
InlineSingleDatePickerInput.propTypes = {
  /**
   * A controlled date value.
   */
  date: momentType,

  /**
   * Called when an open event happens.
   * @param {SyntheticEvent} event - React's original SyntheticEvent.
   * @param {Object} props - All the props.
   * @returns {void}
   */
  onOpen: func,

  /**
   * Called when a close event happens.
   * @param {SyntheticEvent} event - React's original SyntheticEvent.
   * @param {Object} props - All the props.
   * @returns {void}
   */
  onClose: func,

  /**
   * Called when the size of the input changed.
   * @param {{ width: number, height: number }} dimensions - The input dimensions.
   * @returns {void}
   */
  onSizeChange: func,

  /**
   * Called on click.
   * @param {SyntheticEvent} event - React's original SyntheticEvent.
   * @param {Object} props - All the props.
   * @returns {void}
   */
  onClick: func,

  /**
   * Called on keydown.
   * @param {SyntheticEvent} event - React's original SyntheticEvent.
   * @param {Object} props - All the props.
   * @returns {void}
   */
  onKeyDown: func,

  /**
   * Called on focus.
   * @param {SyntheticEvent} event - React's original SyntheticEvent.
   * @param {Object} props - All the props.
   * @returns {void}
   */
  // eslint-disable-next-line react/no-unused-prop-types
  onFocus: func,

  /**
   * Called on blur.
   * @param {SyntheticEvent} event - React's original SyntheticEvent.
   * @param {Object} props - All the props.
   * @returns {void}
   */
  // eslint-disable-next-line react/no-unused-prop-types
  onBlur: func,

  /**
   * Called when a clear event happens.
   * @param {SyntheticEvent} event - React's original SyntheticEvent.
   * @param {Object} props - All the props.
   * @returns {void}
   */
  onClear: func,

  /**
   * The placeholder text.
   */
  placeholder: string,

  /**
   * A format to display a date.
   */
  dateFormat: string,

  /**
   * A custom React SVG component displayed after the date text when the date picker is opened.
   */
  openIcon: node,

  /**
   * A custom React SVG component displayed after the date text when the date picker is closed.
   */
  closeIcon: node,

  /**
   * A custom React SVG component indicating that the date picker is closed when the date picker is disabled.
   */
  disabledCloseIcon: node,

  /**
   * A disabled input does not allow user interaction.
   */
  disabled: bool,

  /**
   * Whether or not the date picker is opened.
   */
  open: bool,

  /**
   * Additional classes.
   */
  className: string
};
InlineSingleDatePickerInput.defaultProps = {
  openIcon: React.createElement(ChevronIcon, {
    className: "rotate-270 fill-primary-500",
    style: {
      width: "1em",
      height: "1em"
    }
  }),
  closeIcon: React.createElement(ChevronIcon, {
    className: "rotate-90 fill-primary-500",
    style: {
      width: "1em",
      height: "1em"
    }
  }),
  disabledCloseIcon: React.createElement(ChevronIcon, {
    className: "rotate-90 fill-cloud-200",
    style: {
      width: "1em",
      height: "1em"
    }
  })
};